/*
 * Created on Oct 27, 2004
 * Copyright (C) Andrea Schweer, 2004
 *
 * This file is part of the Greenstone Alerting Service.
 * Refer to the COPYING file in the base directory of this package
 * for licensing information.
 */
package org.greenstone.gsdlas.profiles;

import java.sql.*;
import java.sql.Connection;
import java.sql.Statement;
import java.util.*;
import java.util.Map;
import java.util.TreeMap;

import org.greenstone.gsdlas.database.DatabaseException;
import org.greenstone.gsdlas.database.DatabaseManager;

/**
 * @author schweer
 * 
 * TODO To change the template for this generated type comment go to Window -
 * Preferences - Java - Code Style - Code Templates
 */
public class Subscription implements Comparable {
    private Map map;
    
    private int id;
    private String username;
    private String name;
    private String email;
    private boolean rssNotification;
    private boolean eventsSincePageNotification;
    
    private int numOfNonEqualsPredicates;
    
    public Subscription(Map valueMap) throws DatabaseException, SQLException {
        map = new TreeMap();
        for (Iterator iter = valueMap.keySet().iterator(); iter.hasNext();) {
            String key = (String) iter.next();
            
            if (!Predicate.isFieldName(key)) {
                continue;
            }
            
            Object value = valueMap.get(key);
            
            Predicate predicate = null;
            if (value instanceof String) {
                predicate = PredicateFactory.createPredicate(key, (String) value);  
                map.put(key, predicate);
            } else if (value instanceof List) {
                List values = (List)value;
                predicate = PredicateFactory.createPredicate(key, values);  
                map.put(key, predicate);
            } 
            
            if (predicate != null && !(predicate instanceof IdEqualsPredicate)) {
                numOfNonEqualsPredicates++;
            }
        }
        
        System.out.println("finished creating predicates");
        
        username = (String) valueMap.get("username");
        name = (String) valueMap.get("subscription_name");
        email = (String) valueMap.get("email");
        rssNotification = valueMap.containsKey("way") && ((List)valueMap.get("way")).contains("rss");
        eventsSincePageNotification = valueMap.containsKey("way") && ((List)valueMap.get("way")).contains("page");
        id = saveToDatabase(true);
        
        for (Iterator iter = getPredicates().iterator(); iter.hasNext();) {
            Predicate predicate = (Predicate) iter.next();
            if (predicate != null) {
                predicate.addSubscription(id);
            }
        }
    }
    
    public boolean containsKey(Object key) {
        return map.containsKey(key);
    }
    
    public boolean containsValue(Object value) {
        return map.containsValue(value);
    }
    
    public Predicate getPredicate(String field) {
        return (Predicate) map.get(field);
    }
    
    public Set keySet() {
        return map.keySet();
    }
    
    public Set getPredicates() {
        Set result = new HashSet();
        for (Iterator iter = map.values().iterator(); iter.hasNext();) {
            Predicate pred = (Predicate) iter.next();
            if (pred != null) {
                result.add(pred);
            }
        }
        return result;
    }
    
    /**
     * @return
     */
    public int getId() {
        return id;
    }
    
    public int getNumOfNonEqualsPredicates() {
        return numOfNonEqualsPredicates;
    }
    
    public String toString() {
        return "Subscription id " + id + ": " + map;
    }
    
    public boolean equals(Object other) {
        if (other == null || !(other instanceof Subscription)) return false;
        return id == ((Subscription)other).id;
    }

    /* (non-Javadoc)
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    public int compareTo(Object arg0) {
        Subscription other = (Subscription) arg0;
        return new Integer(id).compareTo(new Integer(other.id));
    }
    
    private int saveToDatabase(boolean initial) throws DatabaseException, SQLException {
        Connection conn = DatabaseManager.getInstance().getDatabaseConnection();
        Statement statement = conn.createStatement();
        String sqlString;
        if (initial) {
            sqlString = "INSERT INTO subscriptions (name, email, rss, page, user) " +
            		"VALUES ('" + name + "','" + email + "'," + (rssNotification ? 1 : 0) +
            		"," + (eventsSincePageNotification ? 1: 0)+ ", '" + username + "');";
        } else {
            sqlString = "UPDATE subscriptions SET name='" + name + "', email='" +
            		email + "', rss=" + (rssNotification ? 1 : 0) + ",page=" + 
            		(eventsSincePageNotification ? 1 : 0)+ "WHERE id=" + id + ";";
            // cannot change user
        }
        System.out.println(sqlString);
        statement.executeUpdate(sqlString);
        
        sqlString = "SELECT id FROM subscriptions WHERE name like '" + name + 
        			"' AND email like '" + email + "' AND rss=" + (rssNotification ? 1 : 0) 
        			+ " AND page=" + (eventsSincePageNotification ? 1 : 0) + " AND user like '" +
        			username + "';";
        System.out.println(sqlString);
        ResultSet result = statement.executeQuery(sqlString);
        int id;
        if (result.next()) {
            id = result.getInt("id");
        } else {
            throw new DatabaseException("Couldn't save subscription");
        }
        
        if (initial) {
           for (Iterator iter = getPredicates().iterator(); iter.hasNext();) {
               Predicate predicate = (Predicate) iter.next();
               sqlString = "INSERT INTO subs_to_predicates (subscription, predicate) " +
               		"VALUES (" + id + "," + predicate.getID() + ");";
               statement.executeUpdate(sqlString);
           }
        }
        return id;
    }
    
}